Imports System
Imports System.Drawing
Imports System.Drawing.Drawing2D
Imports System.Windows.Forms

Class FirstForm
  Inherits Form

  'Note that if you close this form,
  'you close any instance of the SecondForm class.
  'But if you close an instance of the SecondForm class,
  'the instance of the FirstForm class is *not* closed.

  'Class level declarations
  Dim myButton As Button
  'Note: the textbox must be declared as a Friend.
  'If declared with the Dim keyword, it is private to this class,
  'and so can't be "seen" by the second form.
  Friend f1TextBox As TextBox
  Dim numClicks As Integer
  Dim mylabel As Label

  'Starting point for the program:
  'The Shared keyword allows this routine within the FirstForm class to be accessed
  'even before we've created an object of the FirstForm type.
  Shared Sub Main()
    Application.Run(New FirstForm)
  End Sub

  'Constructor for the class
  Sub New()
    'Identify this form as frm1, allowing "frm1" to be the reference
    'for this form throughout the project.  Note that frm1 is declared
    'in the module, and is declared as Friend.
    frm1 = Me
    Text = "FirstForm"

    'Ensure random numbers are genuinely random
    Randomize()

    'Create the button
    myButton = New Button
    ' "Attach" this button to 'Me', which in this context is this form.
    myButton.Parent = Me
    'ClientSize is the usable area of the form, not including the blue borders
    myButton.Top = CInt(ClientSize.Height * 0.1)
    myButton.Height = CInt(ClientSize.Height * 0.1)
    myButton.Left = CInt(ClientSize.Width * 0.1)
    'Ensure that the font named is one likely to be on the user's system.
    '(Code can be written to check that the font is available.)
    Dim myFont As Font = New Font("Arial", 14, FontStyle.Regular)
    myButton.Font = myFont
    myButton.Text = "Goto Form2"
    myButton.Width = CInt(ClientSize.Width * 0.15)
    'Give the button a "Handler".  When the button is clicked, 
    'the code in the routine "myButtonClick" will be run.
    AddHandler myButton.Click, AddressOf myButtonClick
    'Create a graphics object
    Dim g As Graphics = CreateGraphics()
    ' Get the Size needed to accommodate the formatted Text.
    Dim preferredSize As Size = g.MeasureString(myButton.Text, myFont).ToSize()
    ' Pad the text and resize the control.
    myButton.Size = New Size(preferredSize.Width + 15, preferredSize.Height)
    ' Clean up the Graphics object.
    g.Dispose()

    'Instantiate the text box
    f1TextBox = New TextBox
    ' "Attach" the textbox to this form.
    f1TextBox.Parent = Me
    f1TextBox.Top = myButton.Top + myButton.Height + 10
    f1TextBox.Multiline = True
    f1TextBox.Left = myButton.Left
    f1TextBox.Width = CInt(ClientSize.Width * 0.8)
    f1TextBox.Height = CInt(ClientSize.Height * 0.3)
    f1TextBox.Text = "I am the text from form #1."

    mylabel = New Label
    mylabel.Parent = Me
    mylabel.Top = f1TextBox.Top + f1TextBox.Height + 10
    mylabel.Left = myButton.Left
    mylabel.Width = CInt(ClientSize.Width * 0.8)
    mylabel.Height = CInt(ClientSize.Height * 0.3)
    mylabel.Font = New Font("Tahoma", 10, FontStyle.Italic)
    mylabel.Text = String.Empty
    mylabel.BackColor = Color.BlanchedAlmond

    'Instantiate a new object of the Messenger type, 
    'which has been declared in the module,
    ' so that it can carry messages
    myMessenger = New Messenger

    'Give the messenger a text
    myMessenger.Message = "Hello, I am an object of the Messenger type, and have been given this message for you from form 1."

  End Sub

  Private Sub myButtonClick(ByVal sender As Object, ByVal ea As EventArgs)
    'Count how many times the button has been clicked.
    numClicks += 1
    If numClicks > 1 Then
      myMessage = "You've clicked this button " & numClicks.ToString & " times."
    Else
      myMessage = "You've clicked this button for the first time."
    End If

    'Ensure that a second instance of SecondForm is not created if one already exists.
    If Not (frm2 Is Nothing) Then
      'An object of the SecondForm type has been created already

      If Not frm2.IsDisposed Then  ' ... and has not been disposed of, so the present one can be re-used.
        frm2.WindowState = FormWindowState.Normal
        frm2.BringToFront()
        frm2.Text = "Trying again, eh?"
      Else ' ... but it has been disposed of, so a new instance is needed.  
        '(frm2 now refers to an object that was, but is no longer.) 
        frm2 = New SecondForm
        frm2.Show()
      End If
    Else
      'frm2 refers to nothing, so create an instance of the SecondForm type.
      frm2 = New SecondForm
      frm2.Show()
    End If
  End Sub

  Private Sub FirstForm_Activated(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Activated
    'Using an object of a class such as Messenger declared as friend so that it can be seen
    'throughout the project is another way of passing information between forms.
    mylabel.Text = myMessenger.Message

    Dim oldnumber, newnumber As Integer
    oldnumber = CInt(Val(Microsoft.VisualBasic.Right(myMessenger.Message, 2)))
    newnumber = myRandom.DoRandom
    Do While newnumber = oldnumber
      newnumber = myRandom.DoRandom
    Loop
    myMessenger.Message = "Every time FirstForm is activated, the messenger's number changes: " & newnumber
  End Sub

End Class